' MSMQ.vbs
'
' Create, find or delete a queue in MSMQ
' Existing package must be deleted if it exists to create a new queue with the same name
' This will write entries to the Registry to enable retrieval from another program
'
' Command line parameters:
'	/TASK=<task>			- What task do you want to perform?  CREATE, FIND, DELETE, DETECT
'	/SERVER=<server name>		- optional: (CREATE) Name of server to create queue on.  Local servers are only type supported here.
'	/QUEUENAME=<queue name>	- optional: (CREATE, FIND, DELETE) The name of the queue to run the task on
'	/QUEUEID=<queue's GUID>		- optional: (DELETE, FIND) If you want to delete or find using the GUID instead of QueueName
'	/S				- optional: Silent mode - No window display
'		example:		msmq.vbs /TASK=CREATE /SERVER=THISCOMPUTER /QUEUENAME="Test Queue"
'				msmq.vbs /TASK=FIND /QUEUEID="{5EADC0D0-7182-11CF-A8FF-0020AFB8FB50}"
'				msmq.vbs /TASK=DELETE /QUEUENAME="Test Queue" /S
'				msmq.vbs /TASK=DETECT
'



' All variables must be created explicitly
Option Explicit

' Internal variables used throughout script
Dim	strTask
Dim	strServerName
Dim	strQueueName
Dim	strQueueID
Dim	strRegistryKey
Dim	strRegistryValue
Dim	bSilent

' Run main routine to kick off process

Main

'----------------------------------------------------------------------------
' Main routine
'
Sub Main()

	On Error Resume Next

	bSilent = 0

	' Get command line parameters supplied by the user
	GetCommandLineArgs

	Select Case strTask
		Case "CREATE":
			If MSMQCreateNew() = 0 Then
				If (bSilent = 0) Then MsgBox("Created queue." & strQueueID)
				strRegistryKey = "CREATE"
				strRegistryValue = "0"
			Else
				If (bSilent = 0) Then MsgBox ("Could not create queue.")
				strRegistryKey = "CREATE"
				strRegistryValue = "-1"
			End If
		Case "FIND":
			If MSMQFindDeleteExisting() = 0 Then
				If (bSilent = 0) Then MsgBox ("Found queue.")
				strRegistryKey = "FIND"
				strRegistryValue = "0"
			Else
				If (bSilent = 0) Then MsgBox ("Could not find queue.")
				strRegistryKey = "FIND"
				strRegistryValue = "-1"
			End If
		Case "DELETE":
			If MSMQFindDeleteExisting() = 0 Then
				If (bSilent = 0) Then MsgBox ("Deleted queue.")
				strRegistryKey = "DELETE"
				strRegistryValue = "0"
			Else
				If (bSilent = 0) Then MsgBox ("Did not delete queue.")
				strRegistryKey = "DELETE"
				strRegistryValue = "-1"
			End If
		Case "DETECT":
			If MSMQDetect() = 0 Then
				If (bSilent = 0) Then MsgBox ("Found MSMQ.")
				strRegistryKey = "DETECT"
				strRegistryValue = "0"
			Else
				If (bSilent = 0) Then MsgBox ("Could not find MSMQ.")
				strRegistryKey = "DETECT"
				strRegistryValue = "-1"
			End If
	End Select

	' For writing to the registry.  Comment this line out if you do not want to do this.
	LogResultInRegistry

	WScript.Quit

End Sub

'----------------------------------------------------------------------------
' GetCommandLineArgs routine:
'
Sub GetCommandLineArgs()

	Dim iCount
	Dim iPos
	Dim strArg
	Dim objArgs

	' Get the arguments method of the WScript object   
	Set objArgs = WScript.Arguments

	' Check for command line arguments
	For iCount = 0 To objArgs.Count - 1
		iPos = InStr(1, objArgs(iCount), "=", vbTextCompare)
		If iPos Then
			strArg = Left(UCase(objArgs(iCount)), iPos - 1)
		Else
			strArg = UCase(objArgs(iCount))
		End If

		Select Case strArg
			Case "/TASK":
				strTask = Right(objArgs(iCount), Len(objArgs(iCount)) - iPos)
			Case "/SERVER":
				strServerName = Right(objArgs(iCount), Len(objArgs(iCount)) - iPos)
			Case "/QUEUENAME": 
				strQueueName = Right(objArgs(iCount), Len(objArgs(iCount)) - iPos)
			Case "/QUEUEID":
				strQueueID = Right(objArgs(iCount), Len(objArgs(iCount)) - iPos)
			Case "/S", "/s":
				bSilent = 1
		End Select
	Next

	' Sanity checkes
	' CREATE needs the strServerName
	' CREATE, FIND, DELETE need the strQueueName
	' DETECT doesn't need anything

	Select Case strTask
		Case "CREATE", "FIND", "DELETE", "DETECT" :
			If Not (strTask = "DETECT") Then ' Detect doesn't need any parameters
				If (strServerName = "") And (strTask = "CREATE") Then ' Create is the only one that needs the server
					If (bSilent = 0) Then WScript.Echo "Error:  '/SERVER=<server name>' not specified"
					WScript.Quit (-1)
				End If

				If (strQueueName = "") Then ' Check all other types beside Detect for queue name
					If (strTask = "FIND") Or (strTask = "DELETE") Then
						If (strQueueID = "") Then
							If (bSilent = 0) Then WScript.Echo "Error:  '/QUEUENAME=<queue name>' or '/QUEUEID=<queue guid>' not specified"
							WScript.Quit (-1)
						End If
					Else
						If (bSilent = 0) Then WScript.Echo "Error:  '/QUEUENAME=<queue name>' not specified"
						WScript.Quit (-1)
					End If
				End If
			End If
		Case Else :
			If (bSilent = 0) Then WScript.Echo "Error:  '/TASK=<task>' not specified or improper"
			WScript.Quit (-1)
	End Select

End Sub

'----------------------------------------------------------------------------
' MSMQFindDeleteExisting routine: Find an existing queue with the name of strQueueName
'	strQueueName		- Name of queue to find
'	0	Found
'	-1	Not Found
'
Function MSMQFindDeleteExisting()

	Dim	objMSMQInfos
	Dim	objMSMQInfo
	Dim	objMSMQQuery
	Dim	strResult
	Dim	numReturn
	Dim	strQueueSearch

	numReturn = -1 'Set this first

	' Create the MSMQ objects
	Set objMSMQInfos = CreateObject ("MSMQ.MSMQQueueInfos")
	Set objMSMQInfo = CreateObject ("MSMQ.MSMQQueueInfo")
	Set objMSMQQuery = CreateObject ("MSMQ.MSMQQuery")
		
	' Find the queue
		'These are usual parameters for LookupQueue.  But, don't work.
		'Label := "Test Queue"
		'ServiceTypeGuid := "{5EADC0D0-7182-11CF-A8FF-0020AFB8FB50}"
	Set objMSMQInfos = objMSMQQuery.LookupQueue
	objMSMQInfos.Reset
	Set objMSMQInfo = objMSMQInfos.Next

	' Return a message saying it has found or deleted the queue
	While not objMSMQInfo is Nothing
		If (strQueueName = "") Then ' If queue name is empty, must be queue guid we look for
			strResult = objMSMQInfo.QueueGuid
			strQueueSearch = strQueueID
		Else
			strResult = objMSMQInfo.Label
			strQueueSearch = strQueueName
		End If
		' dbg: MsgBox("Cycling through now strResult = " & strResult)
		If strResult = strQueueSearch Then
			Err.Clear
			Select Case strTask
				Case "FIND":
					' dbg: MsgBox("In FIND section")
					On Error Resume Next
					' We have to double check. Sometimes queues don't get all the way deleted.
					objMSMQInfo.Update
					' If it can't update it, it doesn't exist anymore
					If Err.Number < 0 Then
						If (bSilent = 0) Then MsgBox("Error: " & Err.Description)
					Else
						If (strQueueName = "") Then
							strQueueName = objMSMQInfo.Label
						End If
						numReturn = 0
						' dbg: MsgBox("Found queue: " & strResult)
					End If
				Case "DELETE":
					' dbg: MsgBox("In DELETE section")
					On Error Resume Next
					objMSMQInfo.Delete
					' If there is an error trying to delete it, it doesn't exist anymore
					If Err.Number < 0 Then
						If (bSilent = 0) Then MsgBox("Error: " & Err.Description)
					Else
						If (strQueueName = "") Then
							strQueueName = objMSMQInfo.Label
						End If
						objMSMQInfo.Refresh
						numReturn = 0
						' dbg: MsgBox("This has deleted the queue")
					End If
			End Select
		End If
		Set objMSMQInfo = objMSMQInfos.Next
	Wend

	objMSMQInfos.Reset

	MSMQFindDeleteExisting = numReturn

End Function

'----------------------------------------------------------------------------
' MSMQCreateNew routine: Find an existing queue with the name of strQueueName
'	strQueueName		- Name of queue to find
'	strServerName		- Name of server (Use current machines name.  Hasn't been tested otherwise.)
'	0	Created
'	-1	Not Created
'
Function MSMQCreateNew()

	Dim	objMSMQInfo
	Dim	numReturn

	On Error Resume Next

	numReturn = -1 'Set this first

	Set objMSMQInfo = CreateObject ("MSMQ.MSMQQueueInfo")

	' Set the path name of the queue
	' For a PRIVATE QUEUE eg. iq.PathName = "[SERVER]\PRIVATE$\[QUEUE NAME]"
	objMSMQInfo.PathName = strServerName & "\" & strQueueName
	objMSMQInfo.Label = strQueueName

	' Create the queue.  FORMAT: oMSMQInfo.Create (BOOLIsTransactional, BOOLIsWorldReadable)
	Err.Clear
	objMSMQInfo.Create
	If Err.Number < 0 Then ' Check to see if any errors occured while creating.  If so, return -1.
		If (bSilent = 0) Then MsgBox("Error: " & Err.Description)
		numReturn = -1
		MSMQCreateNew = numReturn
		Exit Function
	End If
	
	strQueueID = objMSMQInfo.QueueGuid

	Err.Clear
	objMSMQInfo.Refresh

	numReturn = 0

	MSMQCreateNew = numReturn

End Function


'----------------------------------------------------------------------------
' MSMQDetect routine: Detect if MSMQ object exists on this machine
'	0	Exists
'	-1	Does not exist
'
Function MSMQDetect()

	Dim	objMSMQInfo
	Dim	numReturn

	On Error Resume Next

	numReturn = -1 'Set this first

	Set objMSMQInfo = CreateObject ("MSMQ.MSMQQueueInfo")

	If Err.Number < 0 Then ' Check to see if any errors occured while creating object
		If (bSilent = 0) Then MsgBox("Error: " & Err.Description)
		numReturn = -1
		MSMQCreateNew = numReturn
		Exit Function
	End If

	numReturn = 0
	MSMQCreateNew = numReturn

End Function


'----------------------------------------------------------------------------
' LogResultInRegistry routine: Log results to the registry so other programs can see results
'
Sub LogResultInRegistry
	Dim objShellObject

	On Error Resume Next

	Set objShellObject = WScript.CreateObject ("WScript.Shell")

	If Not (strQueueName = "") Then
		objShellObject.RegWrite "HKCU\MSMQ Test\" & strRegistryKey & "\" & strQueueName, strRegistryValue
	Else
		objShellObject.RegWrite "HKCU\MSMQ Test\" & strRegistryKey & "\" & strRegistryKey, strRegistryValue
	End If

	Wscript.DisconnectObject objShellObject

End Sub
